import java.awt.image.*;
import java.io.File;
import javax.imageio.ImageIO;


/**
 *
 * @author Marin Ferecatu
 */
public class JPicture {
    static final int RGB_DEPTH = 3, depth = 3, RED = 0, GREEN = 1, BLUE = 2;
    String filename;
    boolean readError;
    int width, height, type;
    float[][][] pixels;

    public JPicture() {
    }

    public JPicture(String fileName) {
        readImageFile(fileName);
    }

    public JPicture(int width, int height, int type, String filename, float[][][] pixels) {
        // deep constructor
        this.width = width;
        this.height = height;
        this.type = type;
        this.filename = filename;
        float[][][] pc = clonePixels(pixels);
        this.pixels = pc;
    }

    public JPicture(int width, int height) {
        this.width = width;
        this.height = height;
        type = 5; // BufferedImage.TYPE_3BYTE_BGR
        pixels = new float[width][height][RGB_DEPTH];
    }

    double max3(double a, double b, double c) {
        return ((a > b) ? ((a > c) ? a : c) : ((b > c) ? b : c));
    }

    double min3(double a, double b, double c) {
        return ((a < b) ? ((a < c) ? a : c) : ((b < c) ? b : c));
    }

    float[][][] clonePixels(float[][][] pixels) {
        int x, y;
        float[][][] pc = new float[width][height][RGB_DEPTH]; // pixels copy
        for(x = 0; x < width; x++) for(y = 0; y < height; y++) {
            System.arraycopy(pixels[x][y], 0, pc[x][y], 0, RGB_DEPTH);
        }
        return pc;
    }

    void setPixels(double[][] v) {
        width = v.length;
        height = v[0].length;
        pixels = new float[width][height][RGB_DEPTH];
        for(int x = 0; x < width; x++) for(int y = 0; y < height; y++) {
            pixels[x][y][RED] = pixels[x][y][GREEN] = pixels[x][y][BLUE] = (float)v[x][y];
        }
    }
    
    void readImageFile(String fileName){
        try {
            BufferedImage img = ImageIO.read(new File(fileName));
            width = img.getWidth();
            height = img.getHeight();
            // type = img.getType();
            type = 5; // BufferedImage.TYPE_3BYTE_BGR
            pixels = new float[width][height][RGB_DEPTH];
            int[] rgbArray = img.getRGB(0, 0, width, height, null, 0, width);
            int pixel;
            for(int x = 0; x < width; ++x)
                for(int y = 0; y < height; ++y) {
                    pixel = rgbArray[y*width + x];
                    // int alpha = (pixel >> 24) & 0xff; // we do not use alpha
                    pixels[x][y][RED]   = (float)((pixel >> 16) & 0xff)/255.0f;
                    if(pixels[x][y][RED] == 1) pixels[x][y][RED] -= 1e-5f;
                    pixels[x][y][GREEN] = (float)((pixel >>  8) & 0xff)/255.0f;
                    if(pixels[x][y][GREEN] == 1) pixels[x][y][GREEN] -= 1e-5f;
                    pixels[x][y][BLUE]  = (float)((pixel      ) & 0xff)/255.0f;
                    if(pixels[x][y][BLUE] == 1) pixels[x][y][BLUE] -= 1e-5f;
                }
        } catch (Exception e) {
            e.printStackTrace();
            readError = true;
        }
    }

    void writeImageFile(String filename) {
        int[] rgbArray = new int[width*height];
        this.filename = filename;
        for(int x = 0; x < width; ++x)
            for(int y = 0; y < height; ++y) {
                rgbArray[y*width + x] = (((int)(pixels[x][y][RED]*255.0f) & 0xff)   << 16) |
                                        (((int)(pixels[x][y][GREEN]*255.0f) & 0xff) <<  8) |
                                        (((int)(pixels[x][y][BLUE]*255.0f) & 0xff)       );
            }
        BufferedImage img = new BufferedImage(width, height, type);
        img.setRGB(0, 0, width, height, rgbArray, 0, width);
        try {
            ImageIO.write(img, "jpg", new File(filename));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    float[] ComputeGrayLevelHistogram(int bins) {
        /*L'histogramme est une liste de 'bins' flottants.*/
        float[] hist = new float[bins];
        /*La variable grey va contenir le calcul du niveau de gris pour chaque pixel.*/
        double grey;

        /*On initialise la liste de l'histogramme à 0 partout.*/
        for (int i = 0 ; i < bins ; i++) hist[i] = 0;

        /*Pour chaque pixel de l'image, on calcule son niveau de gris,
		et on incrémente la case adéquate de l'histogramme. */
        for (int i = 0 ; i < this.width ; i++) {
            for (int j = 0 ; j < this.height ; j++) {
                grey = 0.299*this.pixels[i][j][RED] + 0.587*this.pixels[i][j][GREEN] + 0.114*this.pixels[i][j][BLUE];
				for (int k = 0 ; k < bins ; k++) {
					if ( (grey >= (float)k/bins) && (grey <= (float)(k+1)/bins) ) {
						hist[k] += 1;
					}
				}
            }
        }

		/*On normalise l'histogramme. */
		for (int k = 0 ; k < bins ; k++) {
			hist[k] = (float)hist[k]/(this.width*this.height);
		}

		return hist;
    }

    float[][][] ComputeRGBHistogram(int bins) {
        /*L'histogramme est un tableau en 3 dimensions de flottants.*/
        float[][][] hist = new float[bins][bins][bins];
		/*Ces trois variables contiendront, pour chaque pixel de l'image,
		la valeur des 3 couleurs du pixel*/
		float rpix, gpix, bpix;
        /*Ces variables contiendront, pour chaque pixel, aux 3 coordonnées de la case
        de l'histogramme à incrémenter.*/
        int rcord, gcord, bcord;
		boolean rtest, gtest, btest;
		float normalize = this.height*this.width;

        /*Pour chaque pixel de l'image, on détermine les coordonnées de la case de
        l'histogramme à incrémenter.*/
		for (int i = 0; i < this.width ; i++ ) {
			for (int j = 0; j< this.height ; j++ ) {
				rtest = false;
				gtest = false;
				btest = false;

				rcord = -1;
				gcord = -1;
				bcord = -1;

				rpix = this.pixels[i][j][RED];
				gpix = this.pixels[i][j][GREEN];
				bpix = this.pixels[i][j][BLUE];

				for (int r = 0; r < bins ; r++ ) {
					if ( (rpix >= (float)r/bins) && (rpix <= (float)(r+1)/bins) ){
						rtest = true;
						rcord = r;
					}
				}

				for (int g = 0; g < bins ; g++ ) {
					if ( (gpix >= (float)g/bins) && (gpix <= (float)(g+1)/bins) ){
						gtest = true;
						gcord = g;
					}
				}

				for (int b = 0; b < bins ; b++ ) {
					if ( (bpix >= (float)b/bins) && (bpix <= (float)(b+1)/bins) ){
						btest = true;
						bcord = b;
					}
				}

				if (gtest && btest && rtest){
					hist[rcord][gcord][bcord]+=1/(normalize);
				}
			}
		}
		return hist;
    }

    public static void main(String[] args) {
/*
        //EXEMPLE
    	JPicture p = new JPicture("horse.jpg");

        for(int i = 0; i < p.width; ++i) {
            for(int j = 0; j < p.height/2; ++j) {
                p.pixels[i][j][RED] = 1 - p.pixels[i][j][RED];
                p.pixels[i][j][GREEN] = 1 - p.pixels[i][j][GREEN];
                p.pixels[i][j][BLUE] = 1 - p.pixels[i][j][BLUE];
            }
        }
        p.writeImageFile("horse1.jpg");
*/        
/*
        //EXERCICE DAMIER
        JPicture damier = new JPicture(400, 400);
        for (int i = 0 ; i < damier.width ; i++) {
        	for (int j = 0 ; j < damier.height ; j++) {
        		int red = (i % 2) == (j % 2) ? 1 : 0;
        		damier.pixels[i][j][RED] = red;
        		damier.pixels[i][j][GREEN] = 1-red;
           		damier.pixels[i][j][BLUE] = 0;	
        	}
        }
        damier.writeImageFile("damier.jpg");
*/
/*
        //EXERCICE INTERVERSION
        JPicture interv = new JPicture("../Base10000/images/282038.jpg");
        pixc = interv.clonePixels(interv.pixels);
        for (int i = 0 ; i < interv.width ; i++) {
        	for (int j = 0 ; j < interv.height ; j++) {
        		float red = interv.pixels[i][j][RED];
        		float green = interv.pixels[i][j][GREEN];
        		float blue = interv.pixels[i][j][BLUE];
        		interv.pixels[i][j][RED] = pixc[i][j][GREEN];
        		interv.pixels[i][j][GREEN] = pixc[i][j][BLUE];
           		interv.pixels[i][j][BLUE] = pixc[i][j][RED];	
        	}
        }
        interv.writeImageFile("interv.jpg");
*/

        JPicture pict = new JPicture("horse.jpg");
        pict.ComputeGrayLevelHistogram(10);
        pict.ComputeRGBHistogram(3);
    }
}
