import java.awt.image.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import javax.imageio.ImageIO;
import java.lang.Math;
import java.util.List;
import java.util.ArrayList;

public class QBE {

    public static double dL2(double[] x, double[] y, int n){
        double sum = 0;
        double tmp = 0;

        for (int i = 0; (i < n) ; i++) {
            tmp = (x[i] - y[i]);
            sum += tmp*tmp;
        }

        return Math.sqrt(sum);
    }

    public static void main(String[] args) {
        try {
            /* Traitement des argumetns */

            /*Le 1er argument est le fichier listant les images,
			on en fait une liste*/
            List<String> imgList = Files.readAllLines(Paths.get(args[0]));

            /*Le 2e argument est un ficher de descripteurs,
			on en fait une liste de la même façon*/
            List<String> descFile = Files.readAllLines(Paths.get(args[1]));

            /*Le 3e argument est l'image de la requête */
            String imgRqst = args[2];

            /*Le 4e (et dernier) argument est le nombre k de résultats souhaités.*/
            int nbRes = Integer.parseInt(args[3])+1;

            /*Si L'image n'est pas dans la BDD*/
            if (!imgList.contains(imgRqst)){
                System.out.println("Img Not Found");
                return;
            }


            /* Définition des variables */

            /*L'indice de l'image dans le fichier contenant la liste des images.*/
            int idxRqst = imgList.indexOf(imgRqst);
            /*La distance entre l'image de la requête et toutes les images de la BDD.*/
            double[] dist = new double[10000];
            /*La liste des résultats.*/
            int[] res = new int[nbRes];

            /*On récupère le vecteur descripteur de l'image de requête. (en String)*/
            String[] descVec = descFile.get(idxRqst).split(" ");
            int len = descVec.length;

            /*Le vecteur descripteur de l'image de la requête*/
            double[] x = new double[len];
            /*Le vecteur descripteur de toutes les autres images de la BDD.*/
            double[] y = new double[len];


            /* Traitement */

            /*On crée le vecteur descripteur de l'image de requête.*/
            for (int i=0; i<len ; i++) {
                x[i] = Double.parseDouble(descVec[i]);
            }

            /*On crée le vecteur descripteur de chaque image de la BDD,
            et on calcule la distance à l'image de requête.*/
            for (int i=0;i < 10000; i++) {
                // On crée y.
                descVec = descFile.get(i).split(" ");
                for (int j=0; j<len ; j++) {
                    y[j] = Double.parseDouble(descVec[j]);
                }
                dist[i] = dL2(x,y,len);
            }

            /*dist est la liste de toutes les distances entre les images et la requête.
            On cherche a présent les nbRes plus petites valeurs dans dist
			(autrement dit, les nbRes plus proches images de la requête).*/

            /*On procède au tri de res par rapport aux distances*/

            /*On initialise res*/
            for ( int i = 0 ; i < nbRes ; i++ ) {
                res[i] = i;
            }
			res.sort
			/*On trie le contenu de res en fonction
			des distances des nbRes premières images*/
            for (int i = 0 ; i < nbRes ; i++){
                for (int j = 0 ; j < (nbRes-1) ; j++){
                    if (dist[res[j]] > dist[res[j+1]]){
                        /*Permutation de res[j] et res[j+1].*/
                        res[j] = res[j] + res[j+1];
                        res[j+1] = res[j] - res[j+1];
                        res[j] = res[j] - res[j+1];
                    }
                }
            }

            /*On a maintenant que pour tout i, dist[res[i]] < dist[res[i+1]]
			On parcourt alors toutes les distances 
			pour placer les nbRes plus petites dans res*/
            for ( int i = nbRes ; i < 10000 ; i++) {
                if ( dist[i] < dist[res[(nbRes-1)]] ){
                    res[(nbRes-1)] = i;
                    for (int j = (nbRes-1) ; j > 0 ; j--) {
                        if (dist[res[j]] < dist[res[j-1]]){
                            /*Permutation de res[j] et res[j-1].*/
                            res[j] = res[j] + res[j-1];
                            res[j-1] = res[j] - res[j-1];
                            res[j] = res[j] - res[j-1];
                        }
                    } 
                }
            }

            /* Sorties du programme */

            /*Génération du fichier HTML*/
            String file_name = "[" + args[1].substring(args[1].lastIndexOf(".")+1) + "]" + imgRqst.substring(0, imgRqst.lastIndexOf("."))    + ".html";
            String line;

            PrintWriter writer = new PrintWriter(file_name, "UTF-8");
            writer.println("<html>");
            writer.println("<title>" + imgRqst + "</title>");

            for (int i = 0 ; i < nbRes ; i++ ) {
                System.out.println(res[i] + " " + dist[res[i]]);
                line = "<img src='../Base10000/images/" + imgList.get(res[i]) + "'>";
                writer.println(line);
            }

            writer.println("</html>");
            writer.close();

            System.out.println("Done");
        }

        /*Traitement des erreurs*/
        catch(IllegalArgumentException | IOException ex){
            ex.printStackTrace();
            System.out.println(ex.getMessage());
        }
    }
}
